#******************************************************
# Loads a sequence of object files into Vue 
#
# - objsequenceloader.py
# - By Mark Caldwell
# - Version 0.2.3
# - 8th December 2006
# - Copyright Mark Caldwell 2006
# - Tested with Vue 6 Pre Release
#
# Warning: This script can be memory intensive with
# complex modesls or when importing long sequences.
# Save your Vue scene before running it.
#
# How to use in 5 easy steps
#
# 1. Download this file onto your computer
#
# 2. Place in Directory Containing Object Files named
#    Mesh01001.obj, Mesh01002.obj, Mesh01003.obj and so on
#    for Realflow, somename_00001.obj, somename_00002.obj and so on
#    for Xfrong (The file extension can be any of
#    the types Vue can import and somename will vary)
#    or 1.obj, 2.obj, 3.obj and so on for other
#    types of file import (The file extension can be any of
#    the types Vue can import)
#
# 3. Then run script and supply responses when prompted.
#
# 4. Wait while it loads the objects in
#
# 5. Position the "Imported Object Target" where you want the
#    objects to be placed during rendering.  Scale or rotate this object
#    to scale and rotate the imported objects.  This object won't appear
#    in your pictures.  You can also move where they are put when not used
#    by moving "Imported Object Out of Shot". After moving this render a
#    single frame to change where the objects are placed.
#
#    To run it go to Python -> Run Python Script
#    Then locate the file on your computer
#
#******************************************************

infinity=[0,0,-10000] # The position to place objects when not displayed [x,y,z]
start=[0,0,0] # The position to place object when displayed [x,y,z]

#----------------------------------------------
# End of Configuration: Don't edit after this
#----------------------------------------------

#----------------------------------------------
# Functions
#----------------------------------------------

def FindExtreemVertex (object,pos,highorlow):
    obj=object.ToMesh()
    verticescount=obj.CountMeshVertices()
    vertices=obj.MeshVertices ()

    if highorlow=='high':
        high=vertices[0][pos]
   
        for i in range(0,verticescount):
            vertex=vertices[i]
            if vertex [pos]>high:
               high=vertex[pos]
        return high
    elif highorlow=='low':
        low=vertices[0][pos]
   
        for i in range(0,verticescount):
            vertex=vertices[i]
            if vertex [pos]<low:
               low=vertex[pos]
        return low
    else:
        return 0

def FindExtreem (object,pos,highorlow):
    if object.IsGroup():
        groupobject=object.ToGroup()
        countgroupobjects=groupobject.CountMembers  ()

        for i in range(1,countgroupobjects):   
            fromgroup=groupobject.Member  (i)
            val=FindExtreemVertex (fromgroup,pos,highorlow)

            if i==1:
                valmax=val
            else:
                if val>valmax and highorlow=='high':
                    valmax=val
                elif val<valmax and highorlow=='low':
                    valmax=val
    else:
        val=FindExtreemVertex (object,pos,highorlow)
    return val

def FlickBookCallback(k):
    positionInf=pyObjInf.Position()
    
    for i in range(0,objrecordnumber):
        obj=objectlist[i-1]
        obj.SetPosition(positionInf[0],positionInf[1],positionInf[2])
    currentFrame=int(CurrentFrame())

    currentobjnumber=int(currentFrame//framegap)

    if currentobjnumber>=objrecordnumber:
        currentobjnumber=objrecordnumber-2
        
    if currentobjnumber<0:
        currentobjnumber=0

    obj=objectlist[currentobjnumber]
    position=pyObj.Position()
    rotation=pyObj.GetRotationAngles ()
    scale=pyObj.GetScale()

    x=biggestextremity[0]-extreem[currentobjnumber][0]+position[0]
    y=biggestextremity[1]-extreem[currentobjnumber][1]+position[1]
    z=biggestextremity[2]-extreem[currentobjnumber][2]+position[2]

    if flipyz==1:
        rot=270
    else:
        rot=0

    obj.SetPosition(x,y,z)
    obj.SetRotationAngles (rotation[0]+rot,rotation[1],rotation[2])
    obj.ResizeAxis(scale[0],scale[1],scale[2])

    if drop=='1':
        pyObj.Move(0,0,1000)
        DeselectAll()
        Select(obj)
        Drop()
        pyObj.Move(0,0,-1000)

#----------------------------------------------
# Internal Variables Set Up: Don't alter these
#----------------------------------------------

x=0
objnumber=1
obj=[]
objectlist=[]
extension='obj'
framegap=1
extreem=[]
biggestextremity=[0,0,0]
objrecordnumber=1

#---------------------------------------------------
# Get User Input
#---------------------------------------------------

typeofimport=Prompt('Type of Sequence to Import. \n1: Numbered files \n2: RealFlow3 \n3: RealFlow4\n4: Xfrog','1',true,'Type of Import')

if typeofimport=='2':
    extension='obj'
    filetype=1
    flipyz=2
elif typeofimport=='3':
    extension='obj'
    filetype=2
    flipyz=2
elif typeofimport=='4':
    extension=Prompt('File extension on files \nmaking up sequence','obj',true,'File Extension')
    filestart=Prompt('File name prefix on files \nmaking up sequence','',true,'File Prefix')
    filetype=3
    flipyz=1
else:
    extension=Prompt('File extension on files \nmaking up sequence','obj',true,'File Extension')
    flipyz=Prompt('Switch Y and Z axis \n 1: Yes \n2: No','2',true,'')
    filetype=0

import_type=Prompt('Import whole sequence?\n 1: Yes \n2: No','1',true,'')

if import_type=='2':
    objnumber=Prompt('First frame object number?','1',true,'')
    objnumber=int(objnumber)
    maxobjnumber=Prompt('Last frame object number?','10',true,'')
    maxobjnumber=int(maxobjnumber)
else:
   objnumber=1
   maxobjnumber=10

framegapret=Prompt('Number of frames to wait between loading each object','1',true,'File Extension')
framegap=int(framegapret)

group=Prompt('Group Objects?\n 1: Yes \n2: No','1',true,'')

drop=Prompt('Drop on import \n 1: Yes \n2: No','1',true,'')

align=Prompt('Align on import \n 1: Yes \n2: No','1',true,'')

if align=='1':
    alignx=Prompt('Alignment X \n1: Left \n2: Right','1',true,'')
    if alignx=='1':
        xalign='high'
        biggestextremity[0]=-10000
    else:
        xalign='low'
        biggestextremity[0]=10000
    
    aligny=Prompt('Alignment Y \n 1: Back \n2: Front','1',true,'')
    if aligny=='1':
        yalign='high'
        biggestextremity[1]=-10000
    else:
        yalign='low'
        biggestextremity[1]=10000

    alignz=Prompt('Alignment Z \n 1: Top \n2: Bottom','1',true,'')
                  
    if alignz=='1':
        zalign='high'
        biggestextremity[2]=-10000
    else:
        zalign='low'
        biggestextremity[2]=10000

if group=='1':
    group=1
else:
    group=2

#---------------------------------------------------
# Main Script Body
#---------------------------------------------------

SetCurrentFrame (0)

# Load objects

while obj!=None and objnumber<=maxobjnumber:
    if filetype==1:
        x=3-len(str(objnumber))
        filename='Mesh01'+'0'*x+str(objnumber)+'.'+extension
    elif filetype==2:
        x=5-len(str(objnumber))
        filename='Mesh01'+'0'*x+str(objnumber)+'.'+extension
    elif filetype==3:
        x=5-len(str(objnumber))
        filename=filestart+'0'*x+str(objnumber)+'.'+extension
    else:
        filename=str(objnumber)+'.'+extension
        
    obj=ImportObject(filename)
    
    if obj!=None:
        objnumber=objnumber+1
        objrecordnumber=objrecordnumber+1

        if import_type=='1':
            maxobjnumber=maxobjnumber+1
            
        objectlist.append(obj)
      
        obj.SetPosition(infinity[0],infinity[1],infinity[2])

        if flipyz==1:
            obj.SetRotationAngles (270,0,0)
        
        if align=='1':
            extremityx=FindExtreem (obj,0,xalign)
            if extremityx>biggestextremity[0] and xalign=='high':
                biggestextremity[0]=extremityx
            elif extremityy<biggestextremity[1] and xalign=='low':
                biggestextremity[0]=extremityx
                
            extremityy=FindExtreem (obj,1,yalign)
            if extremityy>biggestextremity[1] and yalign=='high':
                biggestextremity[1]=extremityy
            elif extremityy<biggestextremity[1] and yalign=='low':
                biggestextremity[1]=extremityy
                
            extremityz=FindExtreem (obj,2,zalign)
            if extremityz>biggestextremity[2] and zalign=='high':
                biggestextremity[2]=extremityz
            elif extremityz<biggestextremity[2] and zalign=='low':
                biggestextremity[2]=extremityz
            extreem.append([extremityx,extremityy,extremityz])
        else:
            extreem.append([0,0,0])

if len(objectlist) >0:
    pyObj=AddPythonObject((-5,-5,-5),(5,5,5),true)
    pyObj.SetPosition(0,0,0)
    pyObj.SetInitFrameCallback(FlickBookCallback)
    pyObj.SetName('Imported Object Target')
    pyObjInf=AddPythonObject((-5,-5,-5),(5,5,5),true)
    pyObjInf.SetPosition(infinity[0],infinity[1],infinity[2])
    pyObjInf.SetName('Imported Object Out of Shot')

    # Group imported objects

    if group==1:
        DeselectAll()
        for i in range(0,(objrecordnumber-1)):
            Select(objectlist[i])
        Group()
        DeselectAll()

    Message("Objects Imported","Success")
else:
    Message("No Objects Imported","Failure")


    


